﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Diagnostics;
using FluentSharp.CoreLib;
using System.Text.RegularExpressions;

namespace ReDosUnitTest
{
   [TestClass]
   public class UnitTest1
   {

      [TestMethod]
      public void IsValidInput() // Демонстрация как времето за изпълнение нараства експоненциално спрямо размера на входа
      {

         const string RegExPattern = @"(\w+\d+)+C";

         var stopWatch = new Stopwatch();

         int maxIterations = 25;

         for (var i = 15; i < maxIterations; i++)
         {
            stopWatch.Start();
            var input = i.randomNumbers() + "!";
            Assert.IsFalse(Regex.IsMatch(input, RegExPattern));

            stopWatch.stop();
            stopWatch.Reset();
         }
      }

      [TestMethod]
      public void ValidateEmailAddress() // Зъл регекс, който правилно разпознава верен и грешен вход забива при определен подготвен вход
      {
         const string RegExPattern = @"^([0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*@([0-9a-zA-Z][-.\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})$";

         var validInput = "aaa@test.com";
         Assert.IsTrue(Regex.IsMatch(validInput, RegExPattern));

         var invalidInput = "aaatest.com";
         Assert.IsFalse(Regex.IsMatch(invalidInput, RegExPattern));


         var email = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa!";

         var stopWatch = new Stopwatch();

         stopWatch.Start();

         Assert.IsFalse(Regex.IsMatch(email, RegExPattern));

         stopWatch.stop();

      }


      [TestMethod]
      public void Fuzz_EmailAddress() // Демонстрация на техниката за дебъгване и прогнозиране на уязвимости Fuzzing - тестване със случаен/деформиран вход
      {
         const string RegExPattern = @"[\w-.]{1,}\@([\w]{1,}\.){1,}[a-z]{2,4}$";
         int maxIterations = 10000;

         Assert.IsFalse(Regex.IsMatch("adadadada", RegExPattern));
         Assert.IsTrue(Regex.IsMatch("test@test.com", RegExPattern));

         var stopWatch = new Stopwatch();

         stopWatch.Start();

         for (var i = 0; i < maxIterations; i++)
         {
            var input = (i * 5).randomString();

            stopWatch.Start();

            Regex.IsMatch(input, RegExPattern);

            stopWatch.stop();
            Debug.WriteLine(stopWatch.Elapsed.TotalSeconds);
            Assert.IsTrue(stopWatch.Elapsed.TotalSeconds <= 3);

            stopWatch.Reset();
         }
      }
      [TestMethod]
      public void ValidateEmailAddressTimeOut() // MatchTimeout параметър, който се използва за предотвратяване на проблема с "катастрофалния backtracking"
      {
         const string RegExPattern = @"^([0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*@([0-9a-zA-Z][-.\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})$";

         var email = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa!";

         var stopWatch = new Stopwatch();

         stopWatch.Start();

         try
         {
            Assert.IsFalse(Regex.IsMatch(email, RegExPattern, RegexOptions.IgnoreCase, TimeSpan.FromSeconds(5)));
         }
         catch (Exception ex)
         {
            Assert.IsNotNull(ex);
            Assert.IsInstanceOfType(ex, typeof(RegexMatchTimeoutException));
         }
         stopWatch.stop();

      }
   }
}































